// const { expect } = require('chai');
const assert = require('assert');
const proxyquire = require('proxyquire');

let posts = {};
let pools = {};
const read = (postId) => posts[postId];
const matrixPools = {
  get: (postId) => pools[postId],
};

const computeBatchPost = proxyquire('./compute-batch-post', {
  '../../../util/forum/read': read,
  '../../../util/db': { matrixPools },
});

describe('computeBatchPost', () => {
  it('multiple posts by one author', async () => {
    posts = {
      a: { authors: [{ authorAddress: '0xa1', weightPPM: 1000000 }] },
      b: { authors: [{ authorAddress: '0xa1', weightPPM: 1000000 }] },
      c: { authors: [{ authorAddress: '0xa1', weightPPM: 1000000 }] },
    };
    pools = {
      a: { fee: 100, result: { votePasses: true, quorumMet: true } },
      b: { fee: 100, result: { votePasses: true, quorumMet: true } },
      c: { fee: 100, result: { votePasses: true, quorumMet: true } },
    };

    const { authors, references } = await computeBatchPost(['a', 'b', 'c']);
    assert.deepEqual(authors, [{ authorAddress: '0xa1', weightPPM: 1000000 }]);
    assert.deepEqual(references, []);
  });

  it('posts by different authors', async () => {
    posts = {
      a: { authors: [{ authorAddress: '0xa1', weightPPM: 1000000 }] },
      b: { authors: [{ authorAddress: '0xa2', weightPPM: 1000000 }] },
    };
    pools = {
      a: { fee: 100, result: { votePasses: true, quorumMet: true } },
      b: { fee: 100, result: { votePasses: true, quorumMet: true } },
    };

    const { authors, references } = await computeBatchPost(['a', 'b']);
    assert.deepEqual(authors, [
      { authorAddress: '0xa1', weightPPM: 500000 },
      { authorAddress: '0xa2', weightPPM: 500000 },
    ]);
    assert.deepEqual(references, []);
  });

  it('posts by different authors and pools with different fees', async () => {
    posts = {
      a: { authors: [{ authorAddress: '0xa1', weightPPM: 1000000 }] },
      b: { authors: [{ authorAddress: '0xa2', weightPPM: 1000000 }] },
    };
    pools = {
      a: { fee: 100, result: { votePasses: true, quorumMet: true } },
      b: { fee: 200, result: { votePasses: true, quorumMet: true } },
    };

    const { authors, references } = await computeBatchPost(['a', 'b']);
    assert.deepEqual(authors, [
      { authorAddress: '0xa1', weightPPM: 333334 },
      { authorAddress: '0xa2', weightPPM: 666666 },
    ]);
    assert.deepEqual(references, []);
  });

  it('posts with multiple authors', async () => {
    posts = {
      a: { authors: [{ authorAddress: '0xa1', weightPPM: 500000 }, { authorAddress: '0xa2', weightPPM: 500000 }] },
      b: { authors: [{ authorAddress: '0xa1', weightPPM: 500000 }, { authorAddress: '0xa3', weightPPM: 500000 }] },
    };
    pools = {
      a: { fee: 100, result: { votePasses: true, quorumMet: true } },
      b: { fee: 100, result: { votePasses: true, quorumMet: true } },
    };

    const { authors, references } = await computeBatchPost(['a', 'b']);
    assert.deepEqual(authors, [
      { authorAddress: '0xa1', weightPPM: 500000 },
      { authorAddress: '0xa2', weightPPM: 250000 },
      { authorAddress: '0xa3', weightPPM: 250000 },
    ]);
    assert.deepEqual(references, []);
  });

  it('post with references', async () => {
    posts = {
      a: { authors: [{ authorAddress: '0xa1', weightPPM: 1000000 }] },
      b: { authors: [{ authorAddress: '0xa2', weightPPM: 1000000 }], references: [{ targetPostId: 'a', weightPPM: 500000 }] },
    };
    pools = {
      b: { fee: 100, result: { votePasses: true, quorumMet: true } },
    };

    const { authors, references } = await computeBatchPost(['b']);
    assert.deepEqual(authors, [
      { authorAddress: '0xa2', weightPPM: 1000000 },
    ]);
    assert.deepEqual(references, [{ targetPostId: 'a', weightPPM: 300000 }]);
  });

  it('post with references and pools with different fees', async () => {
    posts = {
      a: { authors: [{ authorAddress: '0xa1', weightPPM: 1000000 }] },
      b: { authors: [{ authorAddress: '0xa2', weightPPM: 1000000 }] },
      c: { authors: [{ authorAddress: '0xa3', weightPPM: 1000000 }], references: [{ targetPostId: 'a', weightPPM: 500000 }] },
      d: { authors: [{ authorAddress: '0xa4', weightPPM: 1000000 }], references: [{ targetPostId: 'b', weightPPM: 500000 }] },
    };
    pools = {
      c: { fee: 100, result: { votePasses: true, quorumMet: true } },
      d: { fee: 200, result: { votePasses: true, quorumMet: true } },
    };

    const { authors, references } = await computeBatchPost(['c', 'd']);
    assert.deepEqual(authors, [
      { authorAddress: '0xa3', weightPPM: 333334 },
      { authorAddress: '0xa4', weightPPM: 666666 },
    ]);
    assert.deepEqual(references, [
      { targetPostId: 'a', weightPPM: 100000 },
      { targetPostId: 'b', weightPPM: 200000 },
    ]);
  });
});
